Page
Library
Module
Module type
Parameter
Class
Class type
Expose more of the underlying protocol buffers machinery to facilitate the iterative use of the CP-SAT solver from OCaml, e.g., to complete hints, or to solve multiple objectives in sequence.
Include the v9.15 Google OR-tools source and build together with the OCaml interface. Rewrite the interface in C++ to avoid an unnecessary copy and allow feasible solution observers.
Remove the eigen3 depext (this template library is now included directly in the build). Adjust other dependencies based on recent work on conf-zlib and conf-bzip2.
Add Sat.Var.any.
Include the v9.15 Google OR-tools source and build together with the OCaml interface. Rewrite the interface in C++ to avoid an unnecessary copy and allow feasible solution observers.
Initial release.
Incomplete interface to the Google OR-Tools CP-SAT solver.