Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Sat.VarSourceRepresentation of integer variables and boolean literals.
A variable or boolean literal. Note that each variable and boolean literal is associated with a specific model. It is an error to mix variables or literals from different models.
Add a new bounded integer variable to a model. For a new variable x, lb <= x <= ub.
Restrict the new bounded integer variable to a domain.
Create a new integer variable constrained to a given value.
Assert that a variable is a boolean variable. Raises Invalid_argument for a variable x that does not satisfy 0 <= x <= 1.
Convert a variable to an integer variable. Raises Invalid_argument on complemented boolean literals.
Pretty-printer for variables and boolean literals.