Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Sat.LinearExprSourceLinear expressions are used in linear constraints and to specify objectives.
A linear expression. An integer offset is maintainted in addition to a list of coefficients and variables. This allows both to normalize boolean literals into positive form and to represent constants (see of_int).
Pretty-printer for linear expressions.